package ListingFieldEditor::CMS;

use strict;

use MT;
use MT::ListProperty;
use MT::Util;
use MT::Util qw( archive_file_for first_n_words encode_html epoch2ts format_ts relative_date encode_url );
use CustomFields::Field;
use CustomFields::App::CMS;
use CustomFields::Util qw( get_meta );

use ListingFieldEditor::CMS::Asset;
use Data::Dumper;

sub init_request {
    my ($eh, $app) = @_;

    my $q = $app->param;
    my $mode = $q->param('__mode');
    return if !($mode eq 'list' || $mode eq 'filtered_list' || $mode eq 'save_from_list');

    my $type = $q->param('_type') || $app->param('datasource');
    return if !($type eq 'entry' || $type eq 'page' || $type eq 'comment' || $type eq 'asset' || $type eq 'member');

    my $blog_id = $q->param('blog_id') || return;

    my $asset_type = [qw( file audio image video)] if $type eq 'asset';

    my @fields = CustomFields::Field->load({
        obj_type => ($asset_type) ? $asset_type : $type eq 'member' ? 'author' : $type,
        blog_id => $blog_id ? [ $blog_id, 0 ] : 0,
    });
    return if !@fields;
    my $prop;
    my @fields_image;
    my $order = 1000;
    foreach my $field (@fields) {
        if (!( $field->type eq 'text' ||
                   $field->type eq 'textarea' ||
                   $field->type eq 'checkbox' ||
                   $field->type eq 'select' ||
                   $field->type eq 'radio' ||
                   $field->type eq 'url' )) {
            push @fields_image, $field;
            next;
        }
        my $key = "blog${blog_id}_".$field->basename;
        $prop->{$key} = {
            label => $field->name,
            display => 'optional',
            html => \&out_html,
            bulk_sort => \&bulk_sort,
            type => $field->type,
            basename => $field->basename,
            order => $order += 100,
            option => $field->options ? ($field->options) : '',
        };
    }
    foreach my $field (@fields_image) {
        next if !( $field->type eq 'image' );
        my $key = "blog${blog_id}_".$field->basename;
        $prop->{$key} = {
            label => $field->name,
            display => 'optional',
            html => \&out_html,
            type => $field->type,
            basename => $field->basename,
            order => $order += 100,
        };
    }

    return 1 if !$prop;
    my $plugin = $app->component('ListingFieldEditor');
    $plugin->registry({
        list_properties => {
            $type => $prop,
        },
    });

    my $plugin = MT->component("ListingFieldEditor");
    my $field_column = $plugin->get_config_value('field_column', 'system');
    if (!$field_column) {
        $plugin->set_config_value('field_column', time(), 'system');
    }

    1;
}

sub bulk_sort {
    my $prop = shift;
    my ($objs) = @_;
    my $basename = $prop->basename;
    my %entry_id = map { $_->id => 1 } @$objs;
    my @entries = MT->model('entry')->load( { id => [ keys %entry_id ] } );
    my %data = map { $_->id => $_->meta('field.' . $basename) } @entries;
    return sort {
        $data{ $a->id } cmp $data{ $b->id }
    } @$objs;
}

sub out_html {
    my ( $prop, $obj ) = @_;

    my $select = MT->translate('Select');
    my $delete = MT->translate('Delete');

    my $basename = $prop->basename;
    my $meta = $obj->meta('field.' . $basename);

    my $meta_data = get_meta($obj);

    unless ( defined( $meta_data->{$basename} ) ) {
        my $field = CustomFields::Field->load(
            {
                blog_id => [ $obj->blog_id, 0 ],
                basename => $basename
            });
        $meta = $field->default;
    }

    if ($prop->type ne 'checkbox') {
        $meta = '---' if !$meta;
    } else {
        $meta = '0' if !$meta;
    }

    $basename =~ s/-/_LFE_/;

    my $id = $obj->id;

    my $asset_id;
    my $orig_meta;

    my $save_changes_label = MT->translate('Save Changes');
    my $save_label = MT->translate('Save');
    my $cancel_label = MT->translate('Cancel');
    my $element;
    if ($prop->type eq 'text') {
        $element = qq{<input type="text" name="note-$basename-$id" id="note-$basename-$id" class="lfetext" value="} . encode_html($meta) . qq{" />};
    } elsif ($prop->type eq 'textarea') {
        $element = qq{<textarea name="note-$basename-$id" id="note-$basename-$id" class="lfetextarea">} . encode_html($meta) . qq{</textarea>};
    } elsif ($prop->type eq 'url') {
        $element = qq{<input type="text" name="note-$basename-$id" id="note-$basename-$id" class="lfeurl" value="} . encode_html($meta) . qq{" />};
    } elsif ($prop->type eq 'checkbox') {
        my $flag = $meta eq '0' ? '' : 'checked="checked"';
        $element = qq{<div><input type="checkbox" name="note-$basename-$id" id="note-$basename-$id" class="lfecheckbox" } . $flag . qq{ /></div>};
    } elsif ($prop->type eq 'select') {
        my @options = split(',', $prop->option);
        my $buf;
        $buf .= qq{<option value=""></option>};
        for my $option (@options) {
            if ( $option =~ /=/ ) {
                my @options = split( /=/, $option );
                if ( $options[0] eq $meta ) {
                    $buf .= qq{<option value="} . $options[0] . qq{" selected="selected">} . $options[1] . qq{</option>};
                    $meta = $options[1];
                } else {
                    $buf .= qq{<option value="} . $options[0] . qq{">} . $options[1] . qq{</option>};
                }
            } else {
                if ( $option eq $meta ) {
                    $buf .= qq{<option value="$option" selected="selected">$option</option>};
                } else {
                    $buf .= qq{<option value="$option">$option</option>};
                }
            }
        }
        $element = qq{<select name="note-$basename-$id" id="note-$basename-$id" class="lfeselect">} . $buf . qq{</select>};
    } elsif ($prop->type eq 'radio') {
        my @options = split(',', $prop->option);
        for my $option (@options) {
            if ( $option eq $meta ) {
                $element .= qq{<input type="radio" name="note-$basename-$id" value="$option" checked="checked" /> } . encode_html($option) . '<br />';
            } else {
                $element .= qq{<input type="radio" name="note-$basename-$id" value="$option" /> } . encode_html($option) . '<br />';
            }
        }
        $element = qq{<div id="note-$basename-$id">} . $element . qq{</div>};
    } elsif ($prop->type eq 'image') {
        $orig_meta = $meta;
        $meta =~ s/.*asset-id="(\d+)".*/$1/;
        my @assets = MT->model('asset')->load( { id => $meta } );
        for (@assets) {
            my ( $url, $w, $h ) = $_->thumbnail_url( Height => 45, Width => 45, Square => 1 );
            $meta = '<img id="image-' . $basename . '-' . $id . '-child" src="'.$url.'" />';
$asset_id = $_->id;
        }
        my $uri = MT->instance->app_uri;
        my $blog_id = MT->instance->blog->id;
        $element = qq{<div id="image-$basename-$id">$meta</div><div class="image-link"><a class="mt-open-dialog" href="${uri}?__mode=list_asset_for_list&_type=asset&blog_id=$blog_id&obj_id=$id&basename=$basename&dialog_view=1&filter=class&filter_val=image&require_type=image&edit_field=customfield_cfimage&asset_select=1'">} . $select . qq{</a> <a href="#$id" class="clear-image" id="delete-$basename-$id">} . $delete . qq{</a></div>};
    }
    my $out;
    if ($prop->type ne 'image') {
        $out = qq{<span id="note-$basename-link-$id"><a href="#$id" class="start-edit" title="$meta">}.encode_html($meta).qq{</a></span><span id="note-$basename-field-$id" style="display: none">} . $element . qq{<span class="buttons"><a href="#$id" title="$save_changes_label" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};
    } else {
        $out = qq{<span id="note-$basename-link-$id"><a href="#$id" class="start-edit" title="">}.$meta.qq{</a></span><span id="note-$basename-field-$id" style="display: none">} . $element . qq{<input class="include_asset_ids-$id" id="include_asset_ids-$basename-$id" type="hidden" value="$asset_id" name="include_asset_ids" /><input id="note-$basename-$id" class="customfield_$basename-$id" type="hidden" value="} . encode_html($orig_meta) . qq{" name="customfield_$basename" /><span class="buttons"><a href="#$id" title="$save_changes_label" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};
    }
    return $out;
}

sub add_script {
    my ($cb, $app, $param, $tmpl) = @_;
    my $type = $app->param('_type');

    my $asset_type = [qw( file audio image video)] if $type eq 'asset';

    my @fields = CustomFields::Field->load({
        obj_type => ($asset_type) ? $asset_type : $type eq 'member' ? 'author' : $type,
        blog_id => $app->blog ? [ $app->blog->id, 0 ] : 0,
    });
    return if !@fields;

#    $type = 'member' if $type eq 'author';


    my $old_buf;
    my $link_buf;
    my $field_buf;
    my $note_buf;
    my $old_note_buf;
    my $empty_note_buf;
    my $empty_old_note_buf;
    my $recover_buf;
    my $clear_note_buf;
    my $new_note_buf;
    my @cancel_buf;
    my $submit_buf;
    my $render_buf;

    my @basenames;
    my $counter = 0;

    my $image_buf;
    my $old_image_buf;
    my $recover_image_buf;
    my $clear_image_buf;

    my $image_asset_id_buf;
    my $old_image_asset_id_buf;
    my $recover_image_asset_id_buf;
    my $clear_image_asset_id_buf;

    my $asset_ids;

    foreach my $field (@fields) {
        next if !( $field->type eq 'text'     ||
                   $field->type eq 'textarea' ||
                   $field->type eq 'checkbox' ||
                   $field->type eq 'select'   ||
                   $field->type eq 'radio'    ||
                   $field->type eq 'url'      ||
                   $field->type eq 'image' );
        ( my $bn = $field->basename ) =~ s/-/_LFE_/;
        $old_buf .=  "var old_note_" . $bn . " = '';\nvar check_note_" . $bn . " = '';\n";
if ( $field->type eq 'image' ) {
        $old_buf .=  "var old_image_asset_id_" . $bn . " = '';\n";
}

        $link_buf .=  "note_links.push( jQuery('#note-" . $bn . "-link-' + id) );\n";
        $field_buf .= "note_fields.push( jQuery('#note-" . $bn . "-field-' + id) );\n";

        # note
        if ($field->type eq 'radio') {
            $note_buf .= "var note_" . $bn . " = jQuery('[name=\"note-" . $bn . "-' + id + '\"]:checked');\n";
        } else {
            $note_buf .= "var note_" . $bn . " = jQuery('#note-" . $bn . "-' + id);\n";
        }

        # old
        if ($field->type eq 'checkbox') {
            $old_note_buf .= "old_note_" . $bn . " =  jQuery('#note-" . $bn . "-' + id).attr('checked');";
        } else {
            $old_note_buf .= "old_note_" . $bn . " = note_" . $bn. ".val();\ncheck_note_" . $bn . " = note_" . $bn . ".val();\n";
        }

        # empty
        if ($field->type ne 'checkbox') {
            $empty_note_buf .= "if (note_" . $bn . ".val() == '---'){\n  note_" . $bn . ".val('');\n  check_note_" . $bn . " = '';\n}\n";
        } else {
            $empty_note_buf .= "if (jQuery('#note-" . $bn . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $bn . "-' + id).attr('checked') == true) { note_" . $bn . "= 1;\ncheck_note_" . $bn . "= 1; } else { note_" . $bn . "= 0;\ncheck_note_" . $bn . "= 0; }";
        }

        # recover
        if ($field->type ne 'checkbox') {
            $recover_buf .= "note_" . $bn . ".val( old_note_" . $bn . " );\n";
        } else {
            $recover_buf .= "note_" . $bn . ".attr('checked', old_note_" . $bn . ");\n";
        }
        $clear_note_buf .=  "old_note_" . $bn . " = '';\n";

        # new
        if ($field->type eq 'checkbox') {
            $new_note_buf .= "var new_note_" . $bn . ";\nif (jQuery('#note-" . $bn . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $bn . "-' + id).attr('checked') == true) { new_note_" . $bn . "= 1; } else { new_note_" . $bn . "= 0; }";
        } else {
            $new_note_buf .= "var new_note_" . $bn . " = note_" . $bn . ".val();\n";
        }

        push @cancel_buf, "check_note_" . $bn . " == new_note_" . $bn;

        # submit
        if ($field->type eq 'radio') {
            $submit_buf .= "var note_" . $bn . " = jQuery('[name=\"note-" . $bn . "-' + id + '\"]:checked').val();\n";
        } elsif ($field->type eq 'checkbox') {
            $submit_buf .= "var note_" . $bn . ";\nif (jQuery('#note-" . $bn . "-' + id)[0]) { if ( jQuery('#note-" . $bn . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $bn . "-' + id).attr('checked') == true) {note_" . $bn . "= 1; } else { note_" . $bn . "= 0; } } else { note_" . $bn . "= undefined; }";
        } else {
            $submit_buf .= "var note_" . $bn . " = jQuery('#note-" . $bn . "-' + id).val();\n";
        }

        # render
        $render_buf .= "customfield_" . $bn . ": note_" . $bn . ",\n";

        # asset_ids
        $asset_ids = "var asset_ids = new Array();\nvar asset_ids_val = '';\njQuery('.include_asset_ids-' + id).each(function(){\nasset_ids[asset_ids.length] = jQuery(this).val();\n});\n
if(asset_ids.length > 1){\nasset_ids_val = '[' + asset_ids.join(',') + ']';\n} else if(asset_ids.length == 1){\nasset_ids_val = asset_ids[0];\n} else {\nasset_ids_val = 0;\n}\n";

        if ($field->type eq 'image') {
            $old_image_buf .= "var old_image_" . $bn . " = '';\n";
            $image_buf .= "old_image_" . $bn . " = jQuery('#image-" . $bn . "-' + id).html();\n";
            $recover_image_buf .= "if(old_note_" . $bn . " == '---') {\nold_note_" . $bn . " = '';\n}\njQuery('#image-" . $bn . "-' + id).html( old_image_" . $bn . " );\nif (old_image_" . $bn . " == '---'){\n  jQuery('#delete-" . $bn . "-' + id).css('display', 'none');\n} else {\n  jQuery('#delete-" . $bn . "-' + id).css('display', 'inline');\n}\n";
            $clear_image_buf .=  "old_image_" . $bn . " = '';\n";

            $image_asset_id_buf .= "old_image_asset_id_" . $bn . " = jQuery('#include_asset_ids-" . $bn . "-' + id).val();\n";
            $recover_image_asset_id_buf .= "jQuery('#include_asset_ids-" . $bn . "-' + id).val( old_image_asset_id_" . $bn . " );\n";
            $clear_image_asset_id_buf .=  "old_image_asset_id_" . $bn . " = '';\n";
        }

        $counter++;
    }
    return 1 if !$counter;
    my $cancel = join(' && ', @cancel_buf);

    my $plugin = $app->component('ListingFieldEditor');
    my $field = $tmpl->getElementById('container');
    my $node = $tmpl->createElement('setvarblock', { name => 'jq_js_include', component => $plugin });

    my $html = <<HTML;
    var current_edit = 0;
    $old_buf
    $old_image_buf

    function clear_image(id, basename_id) {
        basename_id.match(/delete-(.*)/);
        var basename_id = RegExp.\$1;
        jQuery('#image-' + basename_id).html('---');
        jQuery('#note-' + basename_id).val('');
        jQuery('#include_asset_ids-' + basename_id).val('');
        jQuery('#delete-' + basename_id).css('display', 'none');
    }

    function edit_note(id) {
        if (current_edit) return;
        var note_links = new Array();
        $link_buf
        var note_fields = new Array();
        $field_buf
        if (!note_links.length || !note_fields.length) return;
        $note_buf
        current_edit = id;
        $old_note_buf
        $image_buf
        $image_asset_id_buf
        $empty_note_buf

        for ( i = 0; i < $counter; ++i ) {
            note_links[i].hide();
            note_fields[i].show();
        }
        return false;
    }

    function cancel_edit(id) {
        var note_links = new Array();
        $link_buf
        var note_fields = new Array();
        $field_buf
        $note_buf
        if (!note_links.length || !note_fields.length) return;
        $recover_image_buf
        $recover_image_asset_id_buf
        $recover_buf
        $clear_image_buf
        $clear_image_asset_id_buf
        $clear_note_buf
        current_edit = 0;

        for ( i = 0; i < $counter; ++i ) {
            note_links[i].show();
            note_fields[i].hide();
        }
        return false;
    }

    function submit_edit(id) {
        if ((0 != current_edit) && (id != current_edit))
            id = current_edit;
        $note_buf
        $new_note_buf

        if ($cancel) {
            cancel_edit(id);
            return false;
        }
        return submit_edit_form(id);
    }

    function submit_edit_form(id) {
        $submit_buf
        $asset_ids
        renderList({
            __mode: 'save_from_list',
            _type: '$type',
            customfield_beacon: 1,
            $render_buf
            include_asset_ids: asset_ids_val,
            id: id,
            forward_list: 1,
            magic_token: 1
        });
        cancelCreateMode();
        return false;
    }

    function setCreateMode () {
        jQuery('#create-inline-notification').show();
        jQuery('#text').val('').focus();
        jQuery('#content-actions').hide();
    }

    function cancelCreateMode () {
        jQuery('#create-inline-notification').hide();
        jQuery('#create-inline-notification input').mtUnvalidate();
        jQuery('#content-actions').show();

        current_edit = 0;
    }

    jQuery(document).on('click', '.start-edit', function () {
        jQuery(this).attr('href').match(/#(\\d+)/);
        var id = RegExp.\$1;
        cancel_edit(current_edit);
        edit_note(id);
        return false;
    });

    jQuery(document).on('click', '.cancel-edit', function () {
        jQuery(this).attr('href').match(/#(\\d+)/);
        var id = RegExp.\$1;
        cancel_edit(id);
        return false;
    });

    jQuery(document).on('click', '.submit-note', function () {
        jQuery(this).attr('href').match(/#(\\d+)/);
        var id = RegExp.\$1;
        submit_edit(id);
        return false;
    });

    jQuery(window).bind('listReady', function() {
        jQuery('a.mt-open-dialog').mtDialog();

        jQuery('.clear-image').each(function () {
            jQuery(this).attr('href').match(/#(\\d+)/);
            var id = RegExp.\$1;
            jQuery(this).attr('id').match(/delete-(.*)/);
            var basename_id = RegExp.\$1;
            if (jQuery('#image-' + basename_id).html() == '---'){
                jQuery(this).css('display', 'none');
            } else {
                jQuery(this).css('display', 'inline');
            }
        });
    });

    jQuery(document).on('click', '.clear-image', function () {
        jQuery(this).attr('href').match(/#(\\d+)/);
        var id = RegExp.\$1;
        var basename = jQuery(this).attr('id');
        clear_image(id, basename);
        return false;
    });
HTML

    $node->innerHTML($html);
    $tmpl->insertBefore($node, $field);

    my $plugin = MT->component("ListingFieldEditor");
    my $node2 = $tmpl->createElement('setvarblock', { name => 'content_header', append => 1 });
    my $html2 = 'Can use ' . rest_day() . 'days';
    $node2->innerHTML($html2);
    $tmpl->insertBefore($node2, $field);

    1;
}

sub add_css {
    my ($cb, $app, $template) = @_;

    my $q = $app->param;
    my $mode = $q->param('__mode');
    return if !($mode eq 'list' || $mode eq 'filtered_list' || $mode eq 'save_from_list');

    my $type = $q->param('_type');
    return if !($type eq 'entry' || $type eq 'page' || $type eq 'comment' || $type eq 'asset' || $type eq 'member');

    my $blog_id = $app->param('blog_id') || 0;

    my $asset_type = [qw( file audio image video)] if $type eq 'asset';

    my @fields = CustomFields::Field->load({
        obj_type => ($asset_type) ? $asset_type : $type eq 'member' ? 'author' : $type,
        blog_id => $blog_id ? [ $blog_id, 0 ] : 0,
    });
    return if !@fields;

    my $plugin = $app->component('ListingFieldEditor');
    my $text_width = $plugin->get_config_value('text_width', 'blog:'.$app->param('blog_id')) || 250;
    my $textarea_width = $plugin->get_config_value('textarea_width', 'blog:'.$app->param('blog_id')) || 400;
    my $text = $text_width - 15;
    my $textarea = $textarea_width - 20;

    my @text_selectors;
    my @textarea_selectors;
    my @button_selectors;
    my @button_selectors2;

    foreach my $field (@fields) {
        next if !( $field->type eq 'text' ||
                   $field->type eq 'textarea' ||
                   $field->type eq 'checkbox' ||
                   $field->type eq 'select' ||
                   $field->type eq 'radio' ||
                   $field->type eq 'url' ||
                   $field->type eq 'image' );
        ( my $bn = $field->basename ) =~ s/-/_LFE_/;
        if ( $field->type eq 'text' || $field->type eq 'url' ) {
            push @text_selectors, ".col.head.blog" . $blog_id . "_" . $bn;
        } elsif ( $field->type eq 'textarea' ) {
            push @textarea_selectors, ".col.head.blog" . $blog_id . "_" . $bn;
        } elsif ( $field->type eq 'checkbox' || $field->type eq 'radio' || $field->type eq 'select' || $field->type eq 'image' ) {
            push @button_selectors, ".col.head.blog" . $blog_id . "_" . $bn;
            if ( $field->type eq 'radio' ) {
            push @button_selectors2, "td.col.blog" . $blog_id . "_" . $bn . " .buttons";
            }
        }
    }
    my $text_selector = join(",", @text_selectors) . "{width: ${text_width}px;}" if @text_selectors;
    my $textarea_selector = join(",", @textarea_selectors) . "{width: ${textarea_width}px;}" if @textarea_selectors;

    my $button_selector = join(",", @button_selectors) . "{width: 140px;}" if @button_selectors;
    my $button_selector2 = join(",", @button_selectors2) . "{display: block; margin-top: 3px;}" if @button_selectors2;

    my $old;
    my $new;

    $old = <<HTML;
<mt:var name="html_head">
HTML
    $old = quotemeta($old);
    $new = <<HTML;
<style type="text/css">
$text_selector
$textarea_selector
$button_selector
$button_selector2

.lfetext, .lfetextarea, .lfeurl {
    -moz-border-bottom-colors: none;
    -moz-border-image: none;
    -moz-border-left-colors: none;
    -moz-border-right-colors: none;
    -moz-border-top-colors: none;
    -moz-box-sizing: border-box;
    background-color: #FBFBFB;
    border-color: #ADADAD #DCDDDD #DCDDDD #ADADAD;
    border-radius: 3px 3px 3px 3px;
    border-right: 1px solid #DCDDDD;
    border-style: solid;
    border-width: 1px;
    box-shadow: 0 1px 0 #F3F3F3;
    font-size: 13px;
    outline: 0 none;
    padding: 0.2em 0.25em;
    vertical-align: middle;
    margin-bottom: 3px;
}
.lfetext, .lfeurl {
    width: ${text}px;
}
.lfetextarea {
    vertical-align: bottom;
    height: 100px;
    width: ${textarea}px;
}
.lfecheckbox {
    margin-bottom: 3px;
}
.lfeselect {
    margin-bottom: 3px;
}
.lferadio .submit-note, .lferadio .cancel-note {
    margin-bottom: 3px;
}
.image-link {
    margin: 3px 0;
}
</style>
  <mt:var name="html_head">
HTML

    $$template =~ s/$old/$new/;

    $old = <<HTML;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>jqueryui/jquery-ui.js?v=<\$mt:var name="mt_version_id" escape="url"\$>"></script>
HTML
    $old = quotemeta($old);
    $new = <<HTML;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>plugins/ListingFieldEditor/js/jquery-ui.js?v=<\$mt:var name="mt_version_id" escape="url"\$>"></script>
HTML
    $$template =~ s/$old/$new/;

    $old = <<HTML;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>jqueryui/jquery-ui.min.js?v=<\$mt:var name="mt_version_id" escape="url"\$>"></script>
HTML
    $old = quotemeta($old);
    $new = <<HTML;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>plugins/ListingFieldEditor/js/jquery-ui.js?v=<\$mt:var name="mt_version_id" escape="url"\$>"></script>
HTML
    $$template =~ s/$old/$new/;
}

sub delete_js {
    my ($cb, $app, $template) = @_;

    my $q = $app->param;
    my $mode = $q->param('__mode');
    my $type = $q->param('_type');
    my $blog_id = $q->param('blog_id');

    return if $mode ne 'list';
    return if $type ne 'entry' && $type ne 'page';
    return if $blog_id eq 0;

    my $old;
    my $new;

    $old = <<HTML;
\$checked.parents('tr').addClass('selected');
HTML
    $old = quotemeta($old);
    $new = <<HTML;
//\$checked.parents('tr').addClass('selected');
HTML
    $$template =~ s/$old/$new/;
}

sub is_limit {
    return (rest_day() > 0) ? 0 : 1;
}

sub rest_day {
    my $plugin = MT->component("ListingFieldEditor");
    my $field_column = $plugin->get_config_value('field_column', 'system') || '';
    my $duration = time() - $field_column;

    my $rest_day = 31 - int($duration / (24 * 3600));

    return ($rest_day > 0) ? $rest_day : 0;
}

sub save_from_list {
    my $app = shift;

    return $app->forward('filtered_list') if is_limit();

    $app->validate_magic or return;
    my $author = $app->user;
    my $type = $app->param('_type') || 'entry';
    $type = 'author' if $type eq 'member';

    my $class = $app->model($type)
        or return $app->errtrans("Invalid parameter");

    my $perms;
    if ( $type eq 'entry' || $type eq 'page' ) {
        $perms = $app->permissions
            or return $app->permission_denied();
    }

    my $id = $app->param('id');
    return if !$id;
    if ( $type eq 'page' ) {
        return $app->permission_denied()
            unless $perms->can_do('edit_all_pages');
    }

    require MT::Blog;
    my $blog_id = $app->param('blog_id');
    my $blog    = MT::Blog->load($blog_id)
        or return $app->error(
        $app->translate( 'Can\'t load blog #[_1].', $blog_id ) );
    my $obj = $class->load($id)
        || return $app->error(
        $app->translate( "No such [_1].", $class->class_label ) );

    if ($type ne 'author' && $blog->class eq 'blog') {
        return $app->error( $app->translate("Invalid parameter") )
            unless $obj->blog_id == $blog_id;
    }

    if ( $type eq 'entry' ) {
        return $app->permission_denied()
            unless $perms->can_edit_entry( $obj, $author );
    }
#    $orig_obj = $obj->clone;

    my $q = $app->param;
    foreach ( $q->param() ) {
        next if $_ eq 'customfield_beacon';

        if (m/^customfield_(.*?)$/) {
            my $field_name = $1;
            if (m/^customfield_(.+?)_cb_beacon$/) {
            } else {
                my $field = CustomFields::Field->load(
                    {   $blog_id
                        ? ( blog_id => [ $blog_id, 0 ] )
                        : ( blog_id => $blog_id ),
                        basename => $field_name
                    }
                ) or next;
                next if $field->type ne 'image';

                # 摜f[^̏ꍇALACe폜
                if ( $q->param("customfield_$field_name") eq '' ) {
                    my $data = $obj->meta( 'field.' . $field_name );
                    $data =~ s/.*mt:asset-id="(\d+)".*/$1/;
                    my $obj_asset = MT::ObjectAsset->load(
                        {   asset_id  => $data,
                            object_ds => 'entry',
                            object_id => $obj->id
                        }
                    );
                    $obj_asset->remove;
                }
            }
        }
    }

    CustomFields::App::CMS::CMSPostSave_customfield_objs('dummy', $app, $obj);

    # for asset
    require MT::Asset;
    require MT::ObjectAsset;
    my $include_asset_ids = $app->param('include_asset_ids');

    if ($include_asset_ids) {
        my @asset_ids;
        if ($include_asset_ids =~ /\[(.*)\]/) {
            @asset_ids = split( ',', $1 );
        } else {
            push @asset_ids, $include_asset_ids;
        }
        my $obj_assets        = ();
        my @obj_assets        = MT::ObjectAsset->load(
            { object_ds => 'entry', object_id => $obj->id } );
        foreach my $obj_asset (@obj_assets) {
            my $asset_id = $obj_asset->asset_id;
            $obj_assets->{$asset_id} = 1;
        }
        my $seen = ();
        foreach my $asset_id (@asset_ids) {
            my $obj_asset = MT::ObjectAsset->load(
                {   asset_id  => $asset_id,
                    object_ds => 'entry',
                    object_id => $obj->id
                }
            );
            unless ($obj_asset) {
                my $obj_asset = new MT::ObjectAsset;
                $obj_asset->blog_id($blog_id);
                $obj_asset->asset_id($asset_id);
                $obj_asset->object_ds('entry');
                $obj_asset->object_id( $obj->id );
                $obj_asset->save;
            }
            $seen->{$asset_id} = 1;
        }
    }
    return $app->forward('filtered_list');
}

1;
